/*
 * Copyright 2022 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "wWaveTerm.h"

#include "wElement.h"
#include "wCache.h"
#include "wGauss.h"

using namespace waves;
using namespace tbox;

/**
 * @brief Build volume stiffness matrix for the waves equation on one element
 */
Eigen::MatrixXd WaveTerm::buildK(Element const &e, std::vector<double> const &u)
{
    // Get precomputed values
    Cache &cache = e.getVCache();
    Gauss &gauss = cache.getVGauss();

    // Stiffness matrix
    Eigen::MatrixXd K = Eigen::MatrixXd::Zero(e.nodes.size(), e.nodes.size());
    for (size_t k = 0; k < gauss.getN(); ++k)
    {
        // Jacobian inverse and shape functions
        Eigen::Matrix3d const &J = e.getJinv(k);
        Eigen::MatrixXd const &dff = cache.getDsf(k);

        // Elementary stiffness matrix
        K += (J * dff).transpose() * J * dff * gauss.getW(k) * e.getDetJ(k);
    }
    return K;
}

/**
 * @brief Build volume/surface mass matrix for the waves equation on one element
 */
Eigen::MatrixXd WaveTerm::buildM(Element const &e)
{
    // Get precomputed values
    Cache &cache = e.getVCache();
    Gauss &gauss = cache.getVGauss();

    // Mass matrix
    Eigen::MatrixXd M = Eigen::MatrixXd::Zero(e.nodes.size(), e.nodes.size());
    for (size_t k = 0; k < gauss.getN(); ++k)
    {
        // Shape functions
        Eigen::VectorXd const &ff = cache.getSf(k);
        M += ff * ff.transpose() * gauss.getW(k) * e.getDetJ(k);
    }
    return M;
}
